@@ -1,7 +1,7 @@
 package App::BencherUtils;
 
-our $DATE = '2016-10-20'; # DATE
-our $VERSION = '0.18'; # VERSION
+our $DATE = '2016-10-27'; # DATE
+our $VERSION = '0.19'; # VERSION
 
 use 5.010001;
 use strict 'subs', 'vars';
@@ -422,18 +422,33 @@ sub list_bencher_scenario_modules {
             (my $mod_pm = "$mod.pm") =~ s!::!/!g;
             require $mod_pm;
             my $scenario = ${"$mod\::scenario"};
+            my %participant_types;
+            for my $p (@{ $scenario->{participants} }) {
+                for my $t (qw/code code_template fcall_template
+                              cmdline cmdline_template
+                              perl_cmdline perl_cmdline_template
+                             /) {
+                    if ($p->{$t}) {
+                        $participant_types{$t}++;
+                        last;
+                    }
+                }
+            }
             push @res, {
                 name => $scenario_name,
                 summary => $scenario->{summary},
                 num_participants => scalar(@{$scenario->{participants}}),
                 num_datasets => $scenario->{datasets} ?
                     scalar(@{$scenario->{datasets}}) : '-',
+                participant_types => join(", ", sort keys %participant_types),
             };
         } else {
             push @res, $scenario_name;
         }
     }
-    $resmeta = {'table.fields' => [qw/name summary num_participants num_datasets/]}
+    $resmeta = {'table.fields' => [qw/name summary num_participants num_datasets
+                                      participant_types
+                                     /]}
         if $detail;
 
     [200, "OK", \@res, $resmeta];
@@ -441,7 +456,7 @@ sub list_bencher_scenario_modules {
 
 $SPEC{format_bencher_result} = {
     v => 1.1,
-    summary => '',
+    summary => 'Format bencher raw/JSON result',
     args => {
         json => {
             summary => 'JSON data',
@@ -450,6 +465,7 @@ $SPEC{format_bencher_result} = {
             pos => 0,
             cmdline_src => 'stdin_or_file',
         },
+        # XXX allow customizing formatters
     },
 };
 sub format_bencher_result {
@@ -497,6 +513,71 @@ sub chart_bencher_result {
     $view_res ? [500, "Failed"] : [200, "OK"];
 }
 
+$SPEC{bencher_module_startup_overhead} = {
+    v => 1.1,
+    summary => 'Accept a list of module names and '.
+        'perform startup overhead benchmark',
+    description => <<'_',
+
+    % bencher-module-startup-overhead Mod1 Mod2 Mod3
+
+is basically a shortcut for creating a scenario like this:
+
+    {
+        module_startup => 1,
+        participants => [
+            {module=>"Mod1"},
+            {module=>"Mod2"},
+            {module=>"Mod3"},
+        ],
+    }
+
+and running that scenario with `bencher`.
+
+_
+    args => {
+        modules => {
+            'x.name.is_plural' => 1,
+            'x.name.singular' => 'module',
+            schema => ['array*', of=>'perl::modname*'],
+            req => 1,
+            pos => 0,
+            greedy => 1,
+            cmdline_src => 'stdin_or_args',
+        },
+    },
+
+};
+sub bencher_module_startup_overhead {
+    my %args = @_;
+
+    my $mods = $args{modules};
+
+    my $scenario = {
+        module_startup => 1,
+        participants => [],
+        (with_process_size => 1) x ($^O =~ /linux/ ? 1:0),
+    };
+    for my $mod (@$mods) {
+        push @{$scenario->{participants}}, {
+            module => $mod,
+        };
+    }
+
+    require Bencher::Backend;
+    my $res = Bencher::Backend::bencher(
+        action => 'bench',
+        scenario => $scenario,
+    );
+    return $res unless $res->[0] == 200;
+
+    my $r = $args{-cmdline_r};
+    return $res if !$r || $r->{format} && $r->{format} !~ /text/;
+
+    [200, "OK", Bencher::Backend::format_result($res),
+     {'cmdline.skip_format'=>1}];
+}
+
 1;
 # ABSTRACT: Utilities related to bencher
 
@@ -512,7 +593,7 @@ App::BencherUtils - Utilities related to bencher
 
 =head1 VERSION
 
-This document describes version 0.18 of App::BencherUtils (from Perl distribution App-BencherUtils), released on 2016-10-20.
+This document describes version 0.19 of App::BencherUtils (from Perl distribution App-BencherUtils), released on 2016-10-27.
 
 =head1 SYNOPSIS
 
@@ -522,6 +603,8 @@ This distribution includes several utilities:
 
 =over
 
+=item * L<bencher-module-startup-overhead>
+
 =item * L<chart-bencher-result>
 
 =item * L<cleanup-old-bencher-results>
@@ -537,6 +620,47 @@ This distribution includes several utilities:
 =head1 FUNCTIONS
 
 
+=head2 bencher_module_startup_overhead(%args) -> [status, msg, result, meta]
+
+Accept a list of module names and perform startup overhead benchmark.
+
+% bencher-module-startup-overhead Mod1 Mod2 Mod3
+
+is basically a shortcut for creating a scenario like this:
+
+ {
+     module_startup => 1,
+     participants => [
+         {module=>"Mod1"},
+         {module=>"Mod2"},
+         {module=>"Mod3"},
+     ],
+ }
+
+and running that scenario with C<bencher>.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<modules>* => I<array[perl::modname]>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
 =head2 chart_bencher_result(%args) -> [status, msg, result, meta]
 
 Generate chart of bencher result and display it.
@@ -622,6 +746,8 @@ Return value:  (any)
 
 =head2 format_bencher_result(%args) -> [status, msg, result, meta]
 
+Format bencher raw/JSON result.
+
 This function is not exported.
 
 Arguments ('*' denotes required arguments):
